/******************************
 * Written by Steven Marsden
 * Date: July 2010
 * Ryerson University Library
 *****************************/


package com.ryerson.LibMobile;


import java.io.InputStream;
import java.net.HttpURLConnection;
import java.net.URL;
import java.util.LinkedList;
import java.util.Timer;

import android.net.ConnectivityManager;
import android.os.Bundle;
import android.os.Handler;
import android.widget.ArrayAdapter;
import android.widget.ImageView;
import android.widget.ListView;
import android.view.View;
import java.util.ArrayList;

import com.ryerson.LibMobile.R;
import com.ryerson.LibMobile.R.id;
import com.ryerson.LibMobile.R.layout;

import android.app.ListActivity;
import android.app.ProgressDialog;
import android.content.Context;
import android.content.Intent;
import android.util.Log;
import android.view.LayoutInflater;
import android.view.View.OnClickListener;
import android.view.ViewGroup;
import android.widget.TextView;
import android.text.Html;
import android.graphics.Bitmap;
import android.graphics.BitmapFactory;


public class BarcodeResult extends ListActivity{	
	private String ISBN;
    private ProgressDialog m_ProgressDialog = null;
    private ArrayList<XMLEntry> entries = null;
    private XMLAdapter m_adapter;
    private Runnable viewResults;
    private Handler mHandler = new Handler();
    private Task t;
    
    
	public void onCreate(Bundle icicle) {
		super.onCreate(icicle);
		setContentView(R.layout.barcoderesult);
        
		entries = new ArrayList<XMLEntry>();
        this.m_adapter = new XMLAdapter(this, R.layout.row, entries);
        setListAdapter(this.m_adapter);
       
        
        TextView titleBar = (TextView) findViewById(R.id.TextView01);
        titleBar.setText((Html.fromHtml("Ryerson Library<br />Barcode Reader"))); 
      
        m_adapter.setContext(this);
        
        ConnectivityManager cm = (ConnectivityManager) getSystemService(Context.CONNECTIVITY_SERVICE);
        boolean online;
        if(cm.getActiveNetworkInfo()!=null){
        	online = cm.getActiveNetworkInfo().isConnectedOrConnecting();
        }
        else{
        	online = false;
        }

        
        if(online){
        	viewResults = new Runnable(){
        
	            public void run() {
	                getResults(ISBN);
	            }
	        	};
	        Thread thread =  new Thread(null, viewResults, "MagentoBackground");
	        thread.start();
	        m_ProgressDialog = ProgressDialog.show(BarcodeResult.this, "Please wait...", "Retrieving data ...", true);
	        t = new Task(m_ProgressDialog, this, mHandler);
	        mHandler.postDelayed(t, 60*1000); //Timeout after 60 seconds
        }
        else{
        	AlertBox.show("No Internet Connection", "Please check your Network Settings and try again later", this);
        
        }
	}
      
	
	private Runnable returnRes = new Runnable() {
			public void run() {
	            if(entries != null && entries.size() > 0){
	                m_adapter.notifyDataSetChanged();
	                for(int i=0;i<entries.size();i++)
	                m_adapter.add(entries.get(i));
	            }
	            m_ProgressDialog.dismiss();
	            mHandler.removeCallbacks(t);
	            m_adapter.setISBN(ISBN);
	            m_adapter.notifyDataSetChanged();
	        }
	};
	private void getResults(String isbn){
		try{
			entries = new ArrayList<XMLEntry>();
	              
      		//Retrieve the passed in ISBN
      		try{
      			Bundle bundle = getIntent().getExtras();
      			ISBN = bundle.getString("ISBN");
      		}
      		catch(Exception e){
      			AlertBox.show("Error", "Could not read ISBN", this);
      			//End this activity prematurely because of this error
          		finish();
      		}
	        
      		LinkedList<XMLEntry> results = new LinkedList<XMLEntry>();
      		
            //Get XML Results
      		try{
      			results = XMLParser.getEntries(ISBN);
      		}
      		catch(Exception e){
      			AlertBox.show("No Internet Connection", "Please check your Network Settings and try again later.", this);
      			m_ProgressDialog.dismiss();
      		}
      		for(int i=0; i<results.size(); i++){
      				
      				XMLEntry temp = results.get(i);
      				XMLEntry thing = results.get(results.size()-1);
      				//If the last result is googlebooks (it usually is, but might not always exist)
      				if(!thing.getThumbURL().equals("")){
      					temp.setThumbURL(results.get(results.size()-1).getThumbURL());
      				}
      				if(temp.getTitle().equals("")) continue;
      				
      				entries.add(temp);  		
      		}
      	
      		Log.i("ARRAY", ""+ entries.size());

        } 
		catch (Exception e) {
			Log.e("BACKGROUND_PROC", e.getMessage());
		}
		runOnUiThread(returnRes);
    }
	
	protected void onListItemClick(ListView l, View v, int position, long id) {
		

		super.onListItemClick(l, v, position, id);
	}
	
	

}

class XMLAdapter extends ArrayAdapter<XMLEntry> {//implements OnClickListener{

    private ArrayList<XMLEntry> items;
    private Context context;
    private String ISBN;
    TextView loc;

    public XMLAdapter(Context context, int textViewResourceId, ArrayList<XMLEntry> items) {
            super(context, textViewResourceId, items);
            this.items = items;
    }

    public View getView(int position, View convertView, ViewGroup parent) {
            View v = convertView;
            

            
            if (v == null) {
                LayoutInflater vi = (LayoutInflater)getContext().getSystemService(Context.LAYOUT_INFLATER_SERVICE);
                v = vi.inflate(R.layout.row, null);
            }
            XMLEntry o = items.get(position);
           
           // v.setOnClickListener(this);            
            
            if(!o.getThumbURL().equals("")){
            		ImageView thumb = (ImageView) v.findViewById(R.id.bookicon);
            		try{
            			thumb.setImageBitmap(drawable_from_url(o.getThumbURL()));
            		}
            		catch(Exception e){} //Take no action as default generic book will still be visible
            }
            
            if (o != null) {
                    
            		TextView title = (TextView) v.findViewById(R.id.title);
                    TextView avail = (TextView) v.findViewById(R.id.avail);
                    TextView callno = (TextView) v.findViewById(R.id.callno);
                    TextView loc = (TextView) v.findViewById(R.id.location);
             
                    
                    if (title != null) {
                    		title.setText((Html.fromHtml("<b>" + o.getTitle() + "</b>")));
                    		
                    }
                    if(avail != null){
                    	if(!o.getAvail().equals("")){
                    		avail.setText((Html.fromHtml(o.getAvail())));
                    	}
                    }
                    if(callno != null){
                    	if(!o.getCallNo().equals("")){
                    		callno.setText((Html.fromHtml(o.getCallNo())));
                    	}
                    }
                    if(loc != null){
                    	if(!o.getLocation().equals("")){
                    		loc.setText((Html.fromHtml(o.getLocation())));
                    	}
                    }
                    
                    //If these are not set, we can assume that we do not own a copy of this resource
                    if(o.getAvail().equals("") || o.getCallNo().equals("") || o.getLocation().equals("")){
                    	callno.setText(Html.fromHtml("<i>Item not available</i>"));
                    	//loc.setText("Request This!");
                    }
            }
            return v;
    }
    
    
    
    Bitmap drawable_from_url(String url) throws java.net.MalformedURLException, java.io.IOException {
    	Bitmap tempImg;
    	HttpURLConnection connection = (HttpURLConnection)new URL(url).openConnection();
    	connection.setRequestProperty("User-agent","Mozilla/4.0");

    	connection.connect();
    	InputStream input = connection.getInputStream();
    	tempImg = BitmapFactory.decodeStream(input);
    	
    	return tempImg;
    }

    public void setContext(Context c){
    	context = c;
    }
    
    public void setISBN(String i){
    	ISBN = i;
    }
    
    
//	public void onClick(View v) {
//		if(loc.getText().toString().equals("Request This!")){
//			Intent intent = new Intent(context,RMobile.class);
//			
//			//Pass the ISBN to the BarcodeResult form
//			Bundle bundle = new Bundle();
//			bundle.putString("FORMURL","http://news.library.ryerson.ca/api/requestItem/?isbn="+ISBN); 
//			intent.putExtras(bundle);
//			
//			context.startActivity(intent); 
//		}
//	}
    
    
    
    
    
}
