/******************************
 * Written by Steven Marsden
 * Date: July 2010
 * Ryerson University Library
 *****************************/

package com.ryerson.LibMobile;


import com.google.zxing.integration.android.IntentIntegrator;
import com.google.zxing.integration.android.IntentResult;
import  com.ryerson.LibMobile.R;
import com.ryerson.LibMobile.R.id;
import  com.ryerson.LibMobile.R.layout;

import java.io.BufferedReader;
import java.io.FileInputStream;
import java.io.FileOutputStream;
import java.io.InputStreamReader;
import java.io.OutputStreamWriter;
import java.io.Writer;
import java.math.BigInteger;
import java.util.ArrayList;
import java.util.List;

import org.apache.http.HttpResponse;
import org.apache.http.NameValuePair;
import org.apache.http.client.entity.UrlEncodedFormEntity;
import org.apache.http.client.methods.HttpPost;
import org.apache.http.cookie.Cookie;
import org.apache.http.impl.client.BasicResponseHandler;
import org.apache.http.impl.client.DefaultHttpClient;
import org.apache.http.message.BasicNameValuePair;
import android.app.Activity;
import android.app.ProgressDialog;
import android.content.Context;
import android.content.DialogInterface;
import android.content.Intent;
import android.content.DialogInterface.OnDismissListener;
import android.net.ConnectivityManager;
import android.os.Bundle;
import android.view.KeyEvent;
import android.view.View;
import android.webkit.CookieManager;
import android.webkit.WebView;
import android.widget.ImageButton;

public class RMobile extends Activity {

	final private boolean DEBUGMODE = true;
	final private String MOBILEURL = "http://m.library.ryerson.ca/";
	
	WebView mWebView;
	LoginDialog dialog;
	CookieManager cookieManager;
	ProgressDialog pd;
		
	public void onCreate(Bundle savedInstanceState) {
		boolean isIntent = false;
		//Load the template
    	super.onCreate(savedInstanceState);

    	setContentView(R.layout.main);
		CustomWebViewClient client = new CustomWebViewClient(this, RMobile.this);

		dialog = new LoginDialog(this);
		
	    //Enable cookies
		cookieManager = CookieManager.getInstance(); 
	    cookieManager.setAcceptCookie(true); 
		
		//Load the web viewer
		mWebView = (WebView) findViewById(R.id.webview);

		mWebView.getSettings().setJavaScriptEnabled(true);
		mWebView.setWebChromeClient(new CustomChromeClient(this,RMobile.this));
		mWebView.setWebViewClient(client);
		
		if(!isIntent){			
			if(isOnline()) mWebView.loadUrl(MOBILEURL);
			else mWebView.loadUrl("file:///android_asset/noConnection.html");
		}

		//The Back Button
		ImageButton bButton = (ImageButton) findViewById(R.id.back);
		bButton.setOnClickListener(new View.OnClickListener() {
            public void onClick(View v) {
                mWebView.goBack();
            }
        });

		//The Forward Button
		ImageButton fButton = (ImageButton) findViewById(R.id.forward);
        fButton.setOnClickListener(new View.OnClickListener() {
            public void onClick(View v) {
                mWebView.goForward();
            }
        });      
        //The Extra Button
		ImageButton speical = (ImageButton) findViewById(R.id.special);
		speical.setOnClickListener(new View.OnClickListener() {
            public void onClick(View v) {
	           if(!DEBUGMODE){
	        	   try{
	            		IntentIntegrator.initiateScan(RMobile.this, "Install Barcode Scanner?", "This application requires you to install 'Barcode Scanner'. Would you like to install it?", "Yes", "No");
	            	}
	            	catch(Exception e){
	            		//This error is usually handled in IntentIntegrator
	            		AlertBox.show("Error", "Could not access Market", RMobile.this);
	            	}
	           }
	           else{	
	            	//These are used for testing. It brings up a menu which allows for several applications to be chosen
	            	Intent myIntent = new Intent(v.getContext(), ExtraMenu.class);
	                startActivityForResult(myIntent, 0);
	           }
            }
        }); 
    }

    
    public boolean onKeyDown(int keyCode, KeyEvent event) {
        //Catch event 'back' to go to previous page in browser
    	if ((keyCode == KeyEvent.KEYCODE_BACK) && mWebView.canGoBack()) {
            mWebView.goBack();
            return true;
        }
        
        //Make sure the application closes rather then be placed in the background so that
        //it is highly unlikely that session will expire
        if(keyCode == KeyEvent.KEYCODE_HOME || keyCode == KeyEvent.KEYCODE_MENU || keyCode == KeyEvent.KEYCODE_ENDCALL){
        	System.exit(0);
        }
        return super.onKeyDown(keyCode, event);
    }
    
 
    
    //Handle the result of the barcode reader
    protected void onActivityResult(int requestCode, int resultCode, Intent data) {
		switch(requestCode) {
		    case IntentIntegrator.REQUEST_CODE: {
		        if (resultCode != RESULT_CANCELED) {
		        	IntentResult scanResult = IntentIntegrator.parseActivityResult(requestCode, resultCode, data);
			        String result ="";    
		        	if (scanResult != null) { 
		        		result = scanResult.getContents(); 
		        	}
		        	else return;
		  
		        	
		        	result = result.trim();
		        	
		        	//Check to see if it is a url
		        	if(result.trim().matches("^http.*://.*")){
		        		if(isOnline()) mWebView.loadUrl(result.trim());
		        		
		    			else mWebView.loadUrl("file:///android_asset/noConnection.html");
		        	}
		        	//Check to see if it is a ISBN (checks if it is numeric)
		        	else if (result.trim().matches("[0-9]*")){
		        		Intent intent = new Intent(RMobile.this,BarcodeResult.class);
						
		        		//Pass the ISBN to the BarcodeResult form
						Bundle bundle = new Bundle();
						bundle.putString("ISBN",scanResult.getContents()); 
						intent.putExtras(bundle);
						
						startActivity(intent); 
		        	}
		        	//Assume that it must be a string
		        	else{
		        		AlertBox.show("Found Message", result, this);
		        	}
		        }
		    }
		        break;
		}
    }
    
    public boolean isOnline(){
    	ConnectivityManager cm = (ConnectivityManager) getSystemService(Context.CONNECTIVITY_SERVICE);
        boolean online;
        if(cm.getActiveNetworkInfo()!=null){
        	online = cm.getActiveNetworkInfo().isConnectedOrConnecting();
        }
        else{
        	online = false;
        	AlertBox.show("No Internet Connection", "Please check your Network Settings and try again later.", RMobile.this);
        }
        return online;
    }
    
}      



