/******************************
 * Written by Steven Marsden
 * Date: July 2010
 * Ryerson University Library
 *****************************/


package com.ryerson.LibMobile;

import java.io.InputStream;
import java.net.URL;
import java.util.LinkedList;
import javax.xml.parsers.SAXParser;
import javax.xml.parsers.SAXParserFactory;
import org.xml.sax.Attributes;
import org.xml.sax.InputSource;
import org.xml.sax.SAXException;
import org.xml.sax.XMLReader;
import org.xml.sax.helpers.DefaultHandler;


public class XMLParser{

	public static LinkedList<XMLEntry>  getEntries(String ISBN) throws Exception{
			
			SAXParserFactory spf = SAXParserFactory.newInstance();
			SAXParser sp = spf.newSAXParser();

			/* Get the XMLReader of the SAXParser we created. */
			XMLReader r = sp.getXMLReader();
			
			//This handles the xml and populates the entries array
			XMLHandler handler = new XMLHandler();
			
			
			// register event handlers
			r.setContentHandler(handler);
			URL url = new URL("http://news.library.ryerson.ca/api/isbnsearch.php?isbn="+ISBN);
			InputStream stream = url.openStream();
			
			r.parse(new InputSource(stream));
			stream.close();
			
			return handler.getEntries();
		}
}



class XMLHandler extends DefaultHandler{

	private LinkedList<XMLEntry> entries = new LinkedList<XMLEntry>();
	private XMLEntry tempEntry = null;              
	private String currentElement = "";
	
	 public LinkedList<XMLEntry> getEntries(){ 
		 return entries; 
	 }
	
	 public void startElement(String namespaceURI, String name,
				String qName, Attributes atts) throws SAXException {
		
		//Beginning a new Record, so we initialise a new XMLEntry
		if( name.equalsIgnoreCase("RECORD") ) {
	    	   tempEntry = new XMLEntry();
		}
		
		else if( name.equalsIgnoreCase("TITLE") ) {
	           currentElement = "TITLE";
	    }
	
		else if( name.equalsIgnoreCase("LOCATION") ) {
			currentElement = "LOCATION";
		}
	
		else if( name.equalsIgnoreCase("CALLNO") ) {
			currentElement = "CALLNO";
		}
	
		else if( name.equalsIgnoreCase("AVAILABILITY") ) {
			currentElement = "AVAILABILITY";
		}
	  
		else if( name.equalsIgnoreCase("RECORDNO") ) {
			currentElement = "RECORDNO";
		}
	  
		else if( name.equalsIgnoreCase("BIB_KEY") ) {
			currentElement = "BIB_KEY";
		}
	  
		else if( name.equalsIgnoreCase("INFO_URL") ) {
			currentElement = "INFO_URL";
		}
	  
		else if( name.equalsIgnoreCase("PREVIEW_URL") ) {
			currentElement = "PREVIEW_URL";
		}
	  
		else if( name.equalsIgnoreCase("THUMBNAIL_URL") ) {
			currentElement = "THUMBNAIL_URL";
		}
	  
		else if( name.equalsIgnoreCase("PREVIEW") ) {
			currentElement = "PREVIEW";
		}
	  
		else if( name.equalsIgnoreCase("EMBEDDABLE") ) {
			currentElement = "EMBEDDABLE";
		}
		else{
			currentElement = "";
		}
	 }
	

	 public void endElement(String namespaceURI, String name, String qName){
	
		if( name.equalsIgnoreCase("RECORD") ) {
				entries.add(tempEntry);
	   
				//Reset variable so it can handle the next set of 'record'
				tempEntry = null;
		}
	}

	
   public void characters( char ch[], int start, int length ){
	   String value = new String( ch , start , length );

       if(!value.trim().equals("")) {

           if( currentElement.equalsIgnoreCase("TITLE") ) {
        	   tempEntry.setTitle(tempEntry.getTitle()+value);
           }

           else if( currentElement.equalsIgnoreCase("LOCATION") ) {
               tempEntry.setLocation(tempEntry.getLocation()+value);
           }

           else if( currentElement.equalsIgnoreCase("CALLNO") ) {
        	   tempEntry.setCallNo(tempEntry.getCallNo()+value);
           }

           else if( currentElement.equalsIgnoreCase("AVAILABILITY") ) {
        	   tempEntry.setAvail(tempEntry.getAvail()+value);
           }

           else if( currentElement.equalsIgnoreCase("RECORDNO") ) {
        	   tempEntry.setRecordNo(tempEntry.getRecordNo()+value);
           }

           else if( currentElement.equalsIgnoreCase("BIB_KEY") ) {
        	   tempEntry.setBibKey(tempEntry.getBibKey()+value);
           }

           else if( currentElement.equalsIgnoreCase("INFO_URL") ) {
        	   tempEntry.setInfoURL(tempEntry.getInfoURL()+value);
           }

           else if( currentElement.equalsIgnoreCase("PREVIEW_URL") ) {
        	   tempEntry.setPreURL(tempEntry.getPreURL()+value);
           }

           else if( currentElement.equalsIgnoreCase("THUMBNAIL_URL") ) {
        	   tempEntry.setThumbURL(tempEntry.getThumbURL()+value);
           }

           else if( currentElement.equalsIgnoreCase("PREVIEW") ) {
        	   tempEntry.setPreview(tempEntry.getPreview()+value);
           }

           else if( currentElement.equalsIgnoreCase("EMBEDDABLE") ) {
        	   tempEntry.setEmbeddable(tempEntry.getEmbeddable()+value);
           }
       }

   }
}