<?php
	/*************************************
	*  Created by: Steven Marsden        *
	*  Email: steven.marsden@ryerson.ca  *
	*  Created: June 2010                *
	*  Modified: August 6, 2010          *
	*  Ryerson University Library        *
	*************************************/

	//Ensure that an ISBN is given 
	
	//Quit if various conditions are not met
	if(!isset($_GET['isbn'])) die; 											//ISBN not given
	if(strlen($_GET['isbn']) != 10 && strlen($_GET['isbn']) != 13  ) die; 	//Not valid ISBN length (10 or 13 digits only allowed)
	if(!is_numeric(substr($_GET['isbn'], 0, -1))) die; 						//Non-numberic (except last character)
	
	
	define("ISBN", trim($_GET['isbn']));
	header("Content-type: text/xml; charset=utf-8"); //Important that the charset to 'utf-8' is set here, as the browser will assume 'us-ascii' if not defined
	print '<?xml version="1.0" encoding="utf-8"?>' . "\n"; 
	
	$xmldata = file_get_contents("http://xisbn.worldcat.org/webservices/xid/isbn/".ISBN."?method=getEditions&format=xml");
	$xml = new SimpleXMLElement($xmldata);
	
	//Create array to hold all record numbers. This is needed to supress duplicate entries for different ISBN's
	$records = array();
		
	//Create arrays for the data
	$location = array();
	$callNumber = array();
	$availability = array();
	$isbn = array();

	//Get the item Title
	$titleData = file_get_contents("http://xisbn.worldcat.org/webservices/xid/isbn/".ISBN."?method=getMetadata&format=xml&fl=*");
	$titleXML = new SimpleXMLElement($titleData);
	$title = $titleXML->isbn[0]['title'];
	$title = xml_character_encode($title);
	$author = xml_character_encode($titleXML->isbn[0]['author']);

	unset($titleData); unset($titleXML);
		
	foreach($xml->isbn as $similarISBN){
		//For every similar ISBN, we need to check it against our library website to see if we have it and pull the relevant information.
		$html = file_get_contents("http://catalogue.library.ryerson.ca/search/i?SEARCH=".$similarISBN."&SUBMIT=Search");
		
		//Remove all html special symbols such as &nbsp;, &gt;, etc...
		$html = preg_replace("/[&]\S*;/", "", $html);
		$record;

		//Case where isbn returns a Ryerson page displaying several resouces (books, e-material, etc...)
		if(preg_match_all("/record=.[0-9][0-9][0-9][0-9][0-9][0-9][0-9]\">/si", $html, $matches)){ 
			foreach($matches[0] as $floor){
				$record = "b".substr($floor,strpos($floor,"=")+2,-2);
				$records[] = $record;
				$isbn[] = $similarISBN;
			}
		}
		
		//Handles special cases like `http://catalogue.library.ryerson.ca/search/i?SEARCH=0385343663&SUBMIT=Search`
		if(preg_match("/.*<a name='anchor_.*/", $html, $matches)){
			if(!strstr($html,"No matches found;")){
				$newURL = substr($matches[0],8+stripos($matches[0], "a href=\""), (strlen($matches[0]) - stripos($matches[0],"\">",0))*-1);
				$html = file_get_contents("http://catalogue.library.ryerson.ca".$newURL);
				$html = preg_replace("/[&]\S*;/", "", $html);
		
			}
		}
		

		//Get the floor item is located on
		if(preg_match_all("/<!-- field 1 -->.*/", $html, $matches)){ 		
			
			$tempCount=0;
			foreach($matches[0] as $floor){
				$location[] = strip_tags($floor);
				
				//Get the record number for every item, and append a -`num` to it. 
				//Eg, b1837151 would become b1837151-1 to ensure uniqueness in the duplicates filter
				if(preg_match("/id=\"recordnum\" href=\"\/record=.*\"/", $html, $rec)){ 
					$r = substr($rec[0], strrpos($rec[0],"=")+1 , -4)."-".$tempCount;
					$records[] = $r;
					$isbn[] = $similarISBN;
					$tempCount++;			
				}
			}
		}

		//Get the callnumber item is located on
		if(preg_match_all("/<!-- field C -->.*/", $html, $matches)){ 
			foreach($matches[0] as $call){
				$callNumber[] = strip_tags($call);
			}
		}

		//Get the status of item
		if(preg_match_all("/<!-- field % -->.*/", $html, $matches)){ 
			foreach($matches[0] as $status){
				$availability[] = strip_tags($status);
			}
		}
	}
	
	$output=""; $count=0;

	//Get all found records and place them in XML formatting
	$used = array();

	for($i=0; $i<count($callNumber); $i++){
		//Verify all data has been loaded properly. If it isn't, do not display any output for that record
		if(isset($title) && isset($location[$i]) && isset($callNumber[$i]) && isset($availability[$i])){
			if(!in_array(trim($records[$i]),$used)){
				
				//Used to flag a duplicate entry
				$flag=0;
				
				//Verify that the code with a '-' has not already been used
				if((!strstr(trim($records[$i]),"-"))){	
					foreach($used as $r){
						if($records[$i] == substr(trim($r),0,8)) $flag=1;
					}
				}
				if($flag==0){
					$output.= "\t<record>\n";
					if(isset($title)) 					$output.= "\t\t<title>". trim($title) ."</title>\n";
					if(isset($author))					$output.= "\t\t<author>". trim($author) ."</author>\n";
					if(isset($location[$i]))			$output.= "\t\t<location>". trim($location[$i]) ."</location>\n";
					if(isset($callNumber[$i]))			$output.= "\t\t<callNo>". trim($callNumber[$i]) ."</callNo>\n";
					if(isset($availability[$i]))		$output.= "\t\t<availability>". trim($availability[$i]) ."</availability>\n";
					if(isset($isbn[$i]))				$output.= "\t\t<isbn>".trim($isbn[$i])."</isbn>\n";
					if(isset($records[$i]))				$output.= "\t\t<recordno>".trim(substr($records[$i],0,8))."</recordno>\n";
					$output.= "\t</record>\n";
					$used[] = trim($records[$i]);
					$count++;
				}
			}
		}
	}
	$googleISBN = ISBN;
	$googleBooks = file_get_contents("http://books.google.com/books?jscmd=viewapi&bibkeys=".ISBN);
	$googleResult = json_decode(substr(($googleBooks),19,-1));
	$googleResult = (array)$googleResult;
	//print_r($googleResult);
	if(!isset($googleResult[ISBN])){
		//Try the other ISBN's
		foreach($xml->isbn as $i){
			$googleBooks = file_get_contents("http://books.google.com/books?jscmd=viewapi&bibkeys=".$i);
			$googleResult = json_decode(substr(($googleBooks),19,-1));
			$googleResult = (array)$googleResult;
			if(isset($googleResult[(string)$i])){
				$googleISBN = (string)$i;
				break;
			}
		}
	}
	
	if(isset($googleResult[$googleISBN])){
		$googleData = (array)$googleResult[$googleISBN];
		$output.= "\t<record type=\"googlebooks\">\n";
		if(isset($googleData["bib_key"]))			$output.= "\t\t<bib_key>".$googleData["bib_key"]."</bib_key>\n";
		if(isset($googleData["info_url"]))			$output.= "\t\t<info_url>".htmlspecialchars($googleData["info_url"])."</info_url>\n";
		if(isset($googleData["preview_url"]))		$output.= "\t\t<preview_url>".htmlspecialchars($googleData["preview_url"])."</preview_url>\n";
		if(isset($googleData["thumbnail_url"])) 	$output.= "\t\t<thumbnail_url>".htmlspecialchars($googleData["thumbnail_url"])."</thumbnail_url>\n";
		if(isset($googleData["preview"]))			$output.= "\t\t<preview>".$googleData["preview"]."</preview>\n";
		if(isset($googleData["embeddable"]))		$output.= "\t\t<embeddable>".$googleData["embeddable"]."</embeddable >\n";		
		$output.= "\t</record>\n";
		$count++;	
	}
	
	switch($count){
		//Case 1: There are no books relating to that ISBN at Ryerson Library
		case 0:
			//Check to see if the XISBN service returned any records of the given ISBN
			if(!isset($title) || $title==""){
				print "<results count=\"".$count."\" libCount=\"0\" message=\"Unrecognized ISBN\">\n</results>";
				break;
			}
			//break;
		
			//Ryerson does not have the book, and there is no google result, but there is an existing ISBN
			print "<results count=\"".$count."\" libCount=\"0\" message=\"Item not available in Ryerson University Library\">\n";
			print "\t<record>\n";
			if(isset($title))	print "\t\t<title>".$title."</title>";
			if(isset($author))	print "\t\t<author>".$author."</author>";
								print "\t\t<isbn>".ISBN."</isbn>";
			print "\t</record>\n";
			print "</results>";
			break;
		
		//Case 2: Either only a google result, or 1 Library result
		case 1:
			if(strstr($output, "<record type=\"googlebooks\">")){
				$count++;
				print "<results count=\"".$count."\" libCount=\"0\" message=\"Item not available in Ryerson University Library\">\n";
				print "\t<record>\n";
				if(isset($title))		print "\t\t<title>".$title."</title>\n";
				if(isset($author))		print "\t\t<author>".$author."</author>";
										print "\t\t<isbn>".ISBN."</isbn>\n";
				print "\t</record>\n";
				print $output;				
				print "</results>";
				break;
			}
			else{
				print "<results count=\"".$count."\" libCount=\"".$count."\" message=\"ok\">\n";
				print $output;
				print "</results>";
				break;
			}
	
		//Case 3: We have records of that ISBN at Ryerson Library
		default:		
			if(!strstr($output, "<record type=\"googlebooks\">")){
				print "<results count=\"".$count."\" libCount=\"".$count."\" message=\"ok\">\n";
				print $output;
				print "</results>";
				break;
			}
			else{
				print "<results count=\"".$count."\" libCount=\"".($count-1)."\" message=\"ok\">\n";
				print $output;
				print "</results>";
				break;
			}
	}

	//Escape any reserved XML characters
	function xml_character_encode($string, $trans='') {
	  return str_replace ( array ( '&', '"', "'", '<', '>', '?' ), array ( '&amp;' , '&quot;', '&apos;' , '&lt;' , '&gt;', '&apos;' ), $string ); 
	} 
	
	
?>
