//
//  RyersonMobileViewController.m
//  RyersonMobile
//
// ================================================================================================
//  Created by Graham McCarthy gmccarthy@ryerson.ca on 02/11/2011
//  Version 1.0
//  
//  Copyright (c) 2011, Graham McCarthy for the Ryerson University Library & Archives
//  http://www.ryerson.ca/library
//  
//  Permission is hereby granted, free of charge, to any person obtaining a copy
//  of this software and associated documentation files (the "Software"), to deal
//  in the Software without restriction, including without limitation the rights
//  to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
//  copies of the Software, and to permit persons to whom the Software is
//  furnished to do so, subject to the following conditions:
//  
//  The above copyright notice and this permission notice shall be included in
//  all copies or substantial portions of the Software.
//  
//  THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
//  IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
//  FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
//  AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
//  LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
//  OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN
//  THE SOFTWARE.
// ================================================================================================

#import "RyersonMobileViewController.h"


#import <SystemConfiguration/SCNetworkReachability.h>
#include <netinet/in.h>

@implementation RyersonMobileViewController

@synthesize webView, activityIndicator;
@synthesize specialAppsViewController;
@synthesize btnBack, btnForward, btnInteractive;


- (BOOL) connectedToNetwork:(BOOL)displayMessage
{
    // Create zero addy
    struct sockaddr_in zeroAddress;
    bzero(&zeroAddress, sizeof(zeroAddress));
    zeroAddress.sin_len = sizeof(zeroAddress);
    zeroAddress.sin_family = AF_INET;
	
    // Recover reachability flags
    SCNetworkReachabilityRef defaultRouteReachability = SCNetworkReachabilityCreateWithAddress(NULL, (struct sockaddr *)&zeroAddress);
    SCNetworkReachabilityFlags flags;
	
    BOOL didRetrieveFlags = SCNetworkReachabilityGetFlags(defaultRouteReachability, &flags);
    CFRelease(defaultRouteReachability);
	
    if (!didRetrieveFlags)
    {
        //printf("Error. Could not recover network reachability flags\n");
        return NO;
    }
	
    BOOL isReachable = flags & kSCNetworkFlagsReachable;
    BOOL needsConnection = flags & kSCNetworkFlagsConnectionRequired;
	
	if(isReachable && !needsConnection){
		return YES;
	} else {
		if(displayMessage) {
			UIAlertView *myAlert = [[UIAlertView alloc] initWithTitle:@"No Internet Connection" 
														  message:@"Please check your Network Settings and try again later." delegate:self cancelButtonTitle:@"Ok" otherButtonTitles:nil];
			[myAlert show];
			[myAlert release];
		}
		
		return NO;
	}

}



// Implement viewDidLoad to do additional setup after loading the view, typically from a nib.
- (void)viewDidLoad {
    [super viewDidLoad];
	
	//set up the webview	
	webView.delegate = self;
	
	//Load the default background page if you cannot connect to ryerson mobile
	NSString *htmlFile = [[NSBundle mainBundle] pathForResource:@"temp" ofType:@"html"];
	NSData *htmlData = [NSData dataWithContentsOfFile:htmlFile];
	[webView loadData:htmlData MIMEType:@"text/html" textEncodingName:@"UTF-8" baseURL:[NSURL URLWithString:@""]]; 
	
	
	//set up the buttons	
	UIImage *imgBack = [UIImage imageNamed:@"iphone-2.png"];
	[btnBack setImage:imgBack];
	
	
	UIImage *imgForward = [UIImage imageNamed:@"iphone-3.png"];
	[btnForward setImage:imgForward];
	
	UIImage *imgInteractive = [UIImage imageNamed:@"iphone-5.png"];
	[btnInteractive setImage:imgInteractive];
	

	if ([self connectedToNetwork:TRUE]) {
		
			//Library Website URL
			NSString *urlAddress = @"http://m.library.ryerson.ca/";
		
			//Create a URL object.
			NSURL *url = [NSURL URLWithString:urlAddress];
		
			//URL Requst Object
			NSURLRequest *requestObj = [NSURLRequest requestWithURL:url];
		
			//Load the request in the UIWebView.
			[webView loadRequest:requestObj];
	
	}
	 
}




- (NSString* )dataFilePath {
	
	NSArray *paths = NSSearchPathForDirectoriesInDomains(NSDocumentDirectory, NSUserDomainMask, YES);
	NSString *documentsDirectory = [paths objectAtIndex:0];
	
	return [documentsDirectory stringByAppendingPathComponent:@"usrdata.plist"];

}



-(IBAction) goBack:(id)sender {
	[webView goBack];

}

-(IBAction) goForward:(id)sender {
	[webView goForward];
}

-(IBAction) launchSpecialApps:(id)sender {
	
	[self presentModalViewController:specialAppsViewController animated:YES];
}

- (BOOL)webView:(UIWebView*)webView shouldStartLoadWithRequest:(NSURLRequest*)request navigationType:(UIWebViewNavigationType)navigationType {

	if([self connectedToNetwork:TRUE]) {
		return YES;
	}
	
	return NO;

}

-(void)webViewDidStartLoad:(UIWebView *)webView{
	if([self connectedToNetwork:FALSE]) {
		[activityIndicator startAnimating];
	}
}

-(void)webViewDidFinishLoad:(UIWebView *)inWebView{
	[activityIndicator stopAnimating];
	
	// Enable or disable back
	[btnBack setEnabled:[webView canGoBack]];
		
	// Enable or disable forward
	[btnForward setEnabled:[webView canGoForward]];
	
}


/*
// The designated initializer. Override to perform setup that is required before the view is loaded.
- (id)initWithNibName:(NSString *)nibNameOrNil bundle:(NSBundle *)nibBundleOrNil {
    if ((self = [super initWithNibName:nibNameOrNil bundle:nibBundleOrNil])) {
        // Custom initialization
    }
    return self;
}
*/

/*
// Implement loadView to create a view hierarchy programmatically, without using a nib.
- (void)loadView {
}
*/


/*
// Override to allow orientations other than the default portrait orientation.
- (BOOL)shouldAutorotateToInterfaceOrientation:(UIInterfaceOrientation)interfaceOrientation {
    // Return YES for supported orientations
    return (interfaceOrientation == UIInterfaceOrientationPortrait);
}
*/

- (void)didReceiveMemoryWarning {
	// Releases the view if it doesn't have a superview.
    [super didReceiveMemoryWarning];
	
	// Release any cached data, images, etc that aren't in use.
}

- (void)viewDidUnload {
	// Release any retained subviews of the main view.
	// e.g. self.myOutlet = nil;
}


- (void)dealloc {
    [super dealloc];
}

@end
