//
//  SpecialAppsViewController.m
//  RyersonMobile
//
// ================================================================================================
//  Created by Graham McCarthy gmccarthy@ryerson.ca on 02/11/2011
//  Version 1.0
//  
//  Copyright (c) 2011, Graham McCarthy for the Ryerson University Library & Archives
//  http://www.ryerson.ca/library
//  
//  Permission is hereby granted, free of charge, to any person obtaining a copy
//  of this software and associated documentation files (the "Software"), to deal
//  in the Software without restriction, including without limitation the rights
//  to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
//  copies of the Software, and to permit persons to whom the Software is
//  furnished to do so, subject to the following conditions:
//  
//  The above copyright notice and this permission notice shall be included in
//  all copies or substantial portions of the Software.
//  
//  THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
//  IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
//  FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
//  AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
//  LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
//  OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN
//  THE SOFTWARE.
// ================================================================================================

#import "SpecialAppsViewController.h"
#import "TBXML.h"
#import <AudioToolbox/AudioServices.h>

#import <SystemConfiguration/SCNetworkReachability.h>
#include <netinet/in.h>

@implementation SpecialAppsViewController

@synthesize resultText, resultLabel, resultImage, lookupBookIndicator;


- (BOOL) connectedToNetwork:(BOOL)displayMessage
{
    // Create zero addy
    struct sockaddr_in zeroAddress;
    bzero(&zeroAddress, sizeof(zeroAddress));
    zeroAddress.sin_len = sizeof(zeroAddress);
    zeroAddress.sin_family = AF_INET;
	
    // Recover reachability flags
    SCNetworkReachabilityRef defaultRouteReachability = SCNetworkReachabilityCreateWithAddress(NULL, (struct sockaddr *)&zeroAddress);
    SCNetworkReachabilityFlags flags;
	
    BOOL didRetrieveFlags = SCNetworkReachabilityGetFlags(defaultRouteReachability, &flags);
    CFRelease(defaultRouteReachability);
	
    if (!didRetrieveFlags)
    {
        //printf("Error. Could not recover network reachability flags\n");
        return NO;
    }
	
    BOOL isReachable = flags & kSCNetworkFlagsReachable;
    BOOL needsConnection = flags & kSCNetworkFlagsConnectionRequired;
	
	if(isReachable && !needsConnection){
		return YES;
	} else {
		if(displayMessage) {
			UIAlertView *myAlert = [[UIAlertView alloc] initWithTitle:@"No Internet Connection" 
															  message:@"Please check your Network Settings and try again later." delegate:self cancelButtonTitle:@"Ok" otherButtonTitles:nil];
			[myAlert show];
			[myAlert release];
		}
		
		return NO;
	}
    
	//return (isReachable && !needsConnection) ? YES : NO;
}




-(IBAction) back:(id)sender {
	[self dismissModalViewControllerAnimated:YES];
}

-(IBAction) barcodeScanner:(id)sender {
	if ( [self scanningAvailable] ) {

		//ZBarReaderController *reader = [ZBarReaderController new];
		ZBarReaderViewController *reader = [ZBarReaderViewController new];
		reader.readerDelegate = self;
	
		ZBarImageScanner *scanner = reader.scanner;
	
	
		//TODO: (optional) additional reader configuration
		// EXAMPLE: disable rarely used I2/5 to improve performance
		[scanner setSymbology: ZBAR_I25
					   config: ZBAR_CFG_ENABLE
						   to: 0];
	
		AVCaptureVideoDataOutput *videoCapture = (AVCaptureVideoDataOutput *) reader.readerView.captureReader.captureOutput; 
		videoCapture.videoSettings = nil; 
	
		// present and release the controller
		[self presentModalViewController: reader
							animated: YES];
		[reader release];
	} else {
		UIAlertView *myAlert = [[UIAlertView alloc] initWithTitle:@"Scanner not supported" 
														  message:@"Your mobile device does not support an auto focus camera and cannot use the barcode scanning application.  We are sorry for the inconvenience." delegate:self cancelButtonTitle:@"Ok" otherButtonTitles:nil];
		[myAlert show];
		[myAlert release];
	}
	
} 



// ADD: do something with decoded barcode data
- (void) imagePickerController: (UIImagePickerController*) reader didFinishPickingMediaWithInfo: (NSDictionary*) info {
   
    // get the results
    id <NSFastEnumeration> results = [info objectForKey: ZBarReaderControllerResults];
    ZBarSymbol *symbol = nil;
    for(symbol in results)
        // EXAMPLE: just grab the first barcode
        break;
    if(!symbol)
        // this should not occur, but doesn't hurt to check
        return;
	
    // EXAMPLE: put the barcode data in the text view
    //resultText.text = symbol.data;
	[resultLabel setText:[NSString stringWithFormat:@"Barcode: %@", symbol.typeName]];	
	[resultText loadHTMLString:@"Translating barcode..." baseURL:nil ];
	//resultText.text = @"Translating barcode...";
	
	// EXAMPLE: do something useful with the barcode image
    resultImage.image = [info objectForKey: UIImagePickerControllerOriginalImage];
	
	// dismiss the controller (NB: dismiss from the *picker*)
    [reader dismissModalViewControllerAnimated: YES];
	
	
	
	//Get the filename of the sound file:
	NSString *path = [NSString stringWithFormat:@"%@%@", [[NSBundle mainBundle] resourcePath], @"/beep-8.wav"];
	
	//declare a system sound id
	SystemSoundID soundID;
	
	//Get a URL for the sound file
	NSURL *filePath = [NSURL fileURLWithPath:path isDirectory:NO];
	
	//Use audio sevices to create the sound
	AudioServicesCreateSystemSoundID((CFURLRef)filePath, &soundID);
	
	//Use audio services to play the sound
	AudioServicesPlaySystemSound(soundID);
	
	
	NSString *dataFromSymbol = [symbol.data stringByTrimmingCharactersInSet:[NSCharacterSet whitespaceAndNewlineCharacterSet]];
	
	NSRange textRange = [[dataFromSymbol lowercaseString] rangeOfString:[@"http" lowercaseString]];
	

	
	if([self connectedToNetwork:TRUE]) {
		
		//is an ISBN SEARCH
		if( [dataFromSymbol length] == 13 || [dataFromSymbol length] == 10) {
			//eg: http://news.library.ryerson.ca/api/isbnsearch.php?isbn=9781848134379
			
			NSString *rulaISBNSearch = [NSString stringWithFormat:@"http://news.library.ryerson.ca/api/isbnsearch.php?isbn=%@", dataFromSymbol];
			NSURL *rulaISBNSearchurl = [[NSURL alloc] initWithString:rulaISBNSearch];
			
			/*UTF-8*/
			
			NSString *htmlCode = [[NSString alloc] initWithContentsOfURL:rulaISBNSearchurl];
			
			NSString *htmlOutput = @"";
			NSString *bookjacket_url = @"http://news.library.ryerson.ca/api/noimage.png";
			
			NSMutableArray *records = [[NSMutableArray alloc] init];
			
			TBXML *tbxml = [[TBXML alloc] initWithXMLString:htmlCode];
			
			//if is valid XML document
			if (tbxml.rootXMLElement) {
				
				NSString *count = [TBXML valueOfAttributeNamed:@"count" forElement:tbxml.rootXMLElement];
				NSString *libCount = [TBXML valueOfAttributeNamed:@"libCount" forElement:tbxml.rootXMLElement];
				
				//NSLog(@"%@", count);
				//NSLog(@"%@", libCount);
				
				if(([count intValue] + [libCount intValue]) > 0) {
					
					// search for the first author element within the root element's children
					TBXMLElement *record = [TBXML childElementNamed:@"record" parentElement:tbxml.rootXMLElement];
					
					if(record != nil) {
						
						// if an author element was found
						while (record != nil) {
							NSString *type = [TBXML valueOfAttributeNamed:@"type" forElement:record];
							
							if(type == nil) {   //if it is not the google record
								TBXMLElement *xTitle = [TBXML childElementNamed:@"title" parentElement:record];
								TBXMLElement *xLocation = [TBXML childElementNamed:@"location" parentElement:record];
								TBXMLElement *xCallNo = [TBXML childElementNamed:@"callNo" parentElement:record];
								TBXMLElement *xAvailability = [TBXML childElementNamed:@"availability" parentElement:record];
								
								//define the four parameters
								NSString *str_title = @"";
								NSString *str_location = @"";
								NSString *str_callNo = @"";
								NSString *str_availability = @"";
								NSString *str_message = @"";
								
								//validate data before using elements
								if( xTitle != nil ) {
									str_title = [TBXML textForElement:xTitle];
								}
								
								if( xLocation != nil ) {
									str_location = [TBXML textForElement:xLocation];
								}
								
								if (xCallNo != nil ) {
									str_callNo = [TBXML textForElement:xCallNo];
								}
								
								if( xAvailability != nil ) {
									str_availability = [TBXML textForElement:xAvailability];
								}
								
								if ( [str_callNo length] <= 0 || [str_availability length] <= 0) {
									str_message = @"Item not available at the Ryerson University Library &amp; Archives";
								}
								NSMutableDictionary *dict = [NSMutableDictionary dictionaryWithCapacity:4];
								[dict setObject:str_title forKey:@"title"];
								[dict setObject:str_location forKey:@"location"];
								[dict setObject:str_callNo forKey:@"callNo"];
								[dict setObject:str_availability forKey:@"availability"];
								[dict setObject:str_message forKey:@"message"];
								
								// find the next sibling element named "isbn"
								[records addObject:dict];
							} else {
								// this is the google records
								//grab the image and save it for the output screen
								TBXMLElement *thumbnail_url = [TBXML childElementNamed:@"thumbnail_url" parentElement:record];
								if(thumbnail_url != nil) {
									bookjacket_url = [TBXML textForElement:thumbnail_url];
								}
							}
							
							//get the next record
							record = [TBXML nextSiblingNamed:@"record" searchFromElement:record];
						} // end while loop			
						
						//create html content out of the records information
						NSString *imgTag = [NSString stringWithFormat:@"<img src='%@' />", bookjacket_url];
						
						htmlOutput = @"<table>";
						
						for (NSMutableDictionary *d in records) {
							htmlOutput = [htmlOutput stringByAppendingString:[NSString stringWithFormat:@"<tr><td colspan='2'>Title: %@</td></tr><tr><td rowspan='3'>%@</td><td>%@</td></tr><tr><td>%@</td></tr><tr><td>%@ %@</td></tr><tr><td colspan='2'>&nbsp;</td></tr>",
																			  [d objectForKey:@"title"], 
																			  imgTag, 
																			  [d objectForKey:@"location"], 
																			  [d objectForKey:@"callNo"], 
																			  [d objectForKey:@"availability"],
																			  [d objectForKey:@"message"] ] ];
						}
						[resultLabel setText:@"Barcode: ISBN"];
						htmlOutput = [htmlOutput stringByAppendingString:@"</table>"];
					} else {
						[resultLabel setText:@"Barcode: ISBN"];
						//if record is null (no records, not even a google record)
						[resultText loadHTMLString:[NSString stringWithFormat:@"<b> Not available at the Ryerson Library. ISBN: %@ </b>", dataFromSymbol] baseURL:nil ];
					}
					
				} else {
					[resultLabel setText:@"Barcode: not an ISBN"];
					// count and lib count are numbers equal to zero, so not a recognized isbn number
					htmlOutput = [NSString stringWithFormat:@"<b> Not a valid ISBN. </b><br /><br />Output: <br /> %@ ", dataFromSymbol];
				}
				
				
			} // if is valid XML output
			
			NSString *testOutput = htmlOutput;
			
			if([testOutput length] > 0) {
				[resultText loadHTMLString:testOutput baseURL:nil ];
				//[resultText setContentToHTMLString: testOutput];
				//resultText.text = testOutput;
			} else {
				[resultText loadHTMLString:[NSString stringWithFormat:@"<b> Not available at the Ryerson Library. %@ </b>", dataFromSymbol] baseURL:nil ];
				//resultText.text = [NSString stringWithFormat:@"Not available at the Ryerson Library. %@", symbol.data];
			}
			
			
		} else if(textRange.location != NSNotFound) {
			//IS URL ADDRESS
			//Does contain the substring
			NSURL *url = [NSURL URLWithString:dataFromSymbol];
			
			[resultLabel setText:@"Barcode: URL"];
			
			if (![[UIApplication sharedApplication] openURL:url]) {
				[resultText loadHTMLString:[NSString stringWithFormat:@"<b>Output:</b><br /> %@ ", dataFromSymbol] baseURL:nil];
				//resultText.text = symbol.data;
				//NSLog(@"%@%@",@"Failed to open url:",[url description]);
			} else {
				//was a valid URL
				[resultText loadHTMLString:[NSString stringWithFormat:@"<b>URL:</b> <br /> <a href='%@' target='_blank'>%@</a> ", dataFromSymbol, dataFromSymbol] baseURL:nil];
			}
		} else {
			//IS SOME OTHER TEXT
			//
			[resultLabel setText:@"Barcode: TEXT"];
			NSString *newString = [dataFromSymbol stringByReplacingOccurrencesOfString: @"\n" withString: @"<br />"];
			[resultText loadHTMLString:[NSString stringWithFormat:@"<b>Output:</b><br /> %@ ", newString] baseURL:nil];
			//resultText.text = symbol.data;
		}
	} else {
		[resultLabel setText:[NSString stringWithFormat:@"Barcode: network problem"]];
		//could not connect to network
		[resultText loadHTMLString:[NSString stringWithFormat:@"<b>Could not connect to network.  Check Network Settings and try again later.</b>"] baseURL:nil];
	}
	    
	
	
} // end function



- (BOOL) scanningAvailable { 
	BOOL available = [ZBarReaderViewController isSourceTypeAvailable:UIImagePickerControllerSourceTypeCamera]; 

	#if TARGET_IPHONE_SIMULATOR 
		return available; 
	#else 
		if (available) { 
			available = NO; 
			for (AVCaptureDevice *cameraDevice in [AVCaptureDevice devicesWithMediaType:AVMediaTypeVideo]) { 
				if ([cameraDevice isFocusModeSupported:AVCaptureFocusModeAutoFocus]) { 
					available = YES; break; 
				} 
			} 
		} 
	#endif 
	
	return available; 
} 





/*
 // The designated initializer.  Override if you create the controller programmatically and want to perform customization that is not appropriate for viewDidLoad.
 - (id)initWithNibName:(NSString *)nibNameOrNil bundle:(NSBundle *)nibBundleOrNil {
 if ((self = [super initWithNibName:nibNameOrNil bundle:nibBundleOrNil])) {
 // Custom initialization
 }
 return self;
 }
 */



// Implement viewDidLoad to do additional setup after loading the view, typically from a nib.
- (void)viewDidLoad {
    [super viewDidLoad];

	NSString *introText = @"<b>Directions:</b><br /> 1) Press 'Scan Barcode' (this may take a moment).<br />2) Hold camera up to barcode (device will beep when barcode is read). <br /><br /><br /> Interprets QR and ISBN barcodes.  ISBN barcodes are checked against the Ryerson University Library &amp; Archives Catalogue for book availability.  This search may take a few moments. ";
	[resultText loadHTMLString:introText baseURL:nil];

}


/*
// Override to allow orientations other than the default portrait orientation.
- (BOOL)shouldAutorotateToInterfaceOrientation:(UIInterfaceOrientation)interfaceOrientation {
    // Return YES for supported orientations
    return (interfaceOrientation == UIInterfaceOrientationPortrait);
}
*/

- (void)didReceiveMemoryWarning {
    // Releases the view if it doesn't have a superview.
    [super didReceiveMemoryWarning];
    
    // Release any cached data, images, etc that aren't in use.
}

- (void)viewDidUnload {
    [super viewDidUnload];
    // Release any retained subviews of the main view.
    // e.g. self.myOutlet = nil;
}


- (void)dealloc {
    [resultText release];
	
	[super dealloc];
	//[redlineSampleView release];
	
}


@end
